# -*- coding: utf-8 -*-
import django

import pytest
from faker import Faker

from consent_user_information.models import ConsentUserInformation

if django.VERSION >= (1, 10):
    from django.urls import reverse
else:
    from django.core.urlresolvers import reverse


@pytest.mark.django_db
def test_consent_user_information(client):
    fake = Faker()

    client.post(reverse('home'), {'mail': fake.email()})

    consent_user_information = ConsentUserInformation.objects.last()

    assert ConsentUserInformation.objects.count() == 1
    assert consent_user_information.user is None
    assert consent_user_information.ip == '127.0.0.1'
    assert consent_user_information.device == 'Other'
    assert consent_user_information.browser == 'Other'


@pytest.mark.django_db
def test_consent_user_information_login(client, django_user_model):
    fake = Faker()
    username = fake.user_name()
    password = fake.password()
    user = django_user_model.objects.create(username=username)

    user.set_password(password)
    user.save()

    client.login(username=username, password=password)
    client.post(reverse('home'), {'mail': fake.email()})

    consent_user_information = ConsentUserInformation.objects.last()

    assert ConsentUserInformation.objects.count() == 1
    assert consent_user_information.user == user
    assert consent_user_information.ip == '127.0.0.1'
    assert consent_user_information.device == 'Other'
    assert consent_user_information.browser == 'Other'


@pytest.mark.django_db
def test_consent_user_information_mail(client, django_user_model):
    fake = Faker()
    username = fake.user_name()
    password = fake.password()
    mail = fake.email()
    user = django_user_model.objects.create(username=username, email=mail)

    user.set_password(password)
    user.save()

    client.get(reverse('simple'), {'mail': mail})

    consent_user_information = ConsentUserInformation.objects.last()

    assert ConsentUserInformation.objects.count() == 1
    assert consent_user_information.user == user
    assert consent_user_information.ip == '127.0.0.1'
    assert consent_user_information.device == 'Other'
    assert consent_user_information.browser == 'Other'



# -*- coding: utf-8 -*-
from django.contrib import admin

from consent_user_information.models import ConsentUserInformation


class ConsentUserInformationAdmin(admin.ModelAdmin):
    """Admin for consent user information"""
    list_display = ('user', 'ip', 'device', 'browser', 'created_at')
    list_select_related = ('user',)
    readonly_fields = ('user', 'ip', 'device', 'browser', 'created_at')
    search_fields = ['user__email', 'user__first_name', 'user__last_name']

    def has_add_permissions(self, request):
        """Can't add a new entry"""
        return False


admin.site.register(ConsentUserInformation, ConsentUserInformationAdmin)



# -*- coding: utf-8 -*-
import logging

from django.contrib.auth import get_user_model

from ipware import get_client_ip

from consent_user_information.models import ConsentUserInformation
from consent_user_information.compat import is_authenticated

logger = logging.getLogger(__name__)


def create_user_consent_information(request, mail=None):
    """To create an user consent information"""
    if not hasattr(request, 'user_agent'):
        raise ValueError('Check if the middleware is present')

    data = {
        'device': request.user_agent.device.family,
        'browser': request.user_agent.browser.family
    }
    client_ip, is_routable = get_client_ip(request)

    if client_ip is None:
        logger.warning("Unable to get the ip")
    else:
        data['ip'] = client_ip

    if mail:
        data['user'] = get_user_model().objects.get(email=mail)
    else:
        if is_authenticated(request.user):
            data['user'] = request.user

    ConsentUserInformation.objects.create(**data)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.10 on 2018-03-02 10:18
# flake8: noqa
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='ConsentUserInformation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('ip', models.GenericIPAddressField(blank=True, null=True)),
                ('device', models.CharField(blank=True, max_length=100)),
                ('browser', models.CharField(blank=True, max_length=255)),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('user', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# -*- coding: utf-8 -*-
import django

import pytest

from consent_user_information.utils import create_user_consent_information

if django.VERSION >= (1, 10):
    from django.urls import reverse
else:
    from django.core.urlresolvers import reverse


def test_without_user_agent(rf):
    request = rf.get(reverse('home'))

    with pytest.raises(ValueError):
        create_user_consent_information(request)



# -*- coding: utf-8 -*-
from consent_user_information.utils import create_user_consent_information


class UserConsentUserInformationMixin(object):
    """A mixin to manage consent user information"""
    def form_valid(self, form):
        """Create the consent user information mixin if the form is valid"""
        create_user_consent_information(self.request)
        return super(UserConsentUserInformationMixin, self).form_valid(form)



# -*- coding: utf-8
from __future__ import unicode_literals, absolute_import
import os

import django

BASE_DIR = os.path.dirname(os.path.dirname(__file__))
DEBUG = True
USE_TZ = True
TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'tests/templates')],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "&vd4&(kcl(oix+x=bdn$2_0mq5313n$*u^bcqw=fkp(x+1v%m)"

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": ":memory:",
    }
}

ROOT_URLCONF = "tests.urls"

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.sites",
    "django_user_agents",
    "consent_user_information",
]

SITE_ID = 1

if django.VERSION >= (1, 10):
    MIDDLEWARE = (
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django_user_agents.middleware.UserAgentMiddleware'
    )
else:
    MIDDLEWARE_CLASSES = (
        'django.contrib.sessions.middleware.SessionMiddleware',
        'django.contrib.auth.middleware.AuthenticationMiddleware',
        'django_user_agents.middleware.UserAgentMiddleware'
    )



# -*- coding: utf-8 -*-
from django.conf.urls import url

from tests.views import home_view, confirmation_view, simple_view

urlpatterns = [
    url(r'^$', home_view, name='home'),
    url(r'^confirmation/$', confirmation_view, name='confirmation'),
    url(r'^simple/$', simple_view, name='simple')
]



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This file is part of MahewinHubStar.
# https://github.com/hobbestigrou/MahewinHubStar

# Licensed under the MIT license:
# http://www.opensource.org/licenses/MIT-license
# Copyright (c) 2015, Hobbestigrou <hobbestigrou@erakis.eu>

from setuptools import setup, find_packages

setup(
    name='django-consent-user-information',
    version='0.2',
    description='To store information about the user.',
    long_description='''
To store information about the user like the browser and device.
''',
    keywords='django, consent',
    author='Hobbestigrou',
    author_email='hobbestigrou@erakis.eu',
    url='https://github.com/hobbestigrou/django-consent-user-information',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Operating System :: OS Independent',
    ],
    packages=find_packages(),
    include_package_data=False,
    install_requires=[
        'django-ipware', 'django-user-agents'
    ],
    entry_points={
        'console_scripts': [
        ],
    },
)



# -*- coding: utf-8 -*-
import django


def is_authenticated(user):
    """To check if the user is authenticated"""
    if django.VERSION >= (1, 10):
        return user.is_authenticated
    else:
        return user.is_authenticated()



# -*- coding: utf-8 -*-
from django.db import models
from django.conf import settings


class AbstractConsentUserInformation(models.Model):
    """An abstract model to extend easily the consent user information"""
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL, null=True, blank=True,
        on_delete=models.CASCADE)
    ip = models.GenericIPAddressField(null=True, blank=True)
    device = models.CharField(max_length=100, blank=True)
    browser = models.CharField(max_length=255, blank=True)
    created_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        abstract = True


class ConsentUserInformation(AbstractConsentUserInformation):
    """To store content user information"""
    pass



# -*- coding: utf-8 -*-
import django
from django.views.generic import FormView, TemplateView

from consent_user_information.views import UserConsentUserInformationMixin
from consent_user_information.utils import create_user_consent_information
from tests.forms import MailForm

if django.VERSION >= (1, 10):
    from django.urls import reverse
else:
    from django.core.urlresolvers import reverse


class HomeView(UserConsentUserInformationMixin, FormView):
    """To display home"""
    template_name = 'simple/home.html'
    form_class = MailForm

    def get_success_url(self):
        """To get the url"""
        return reverse('confirmation')


class ConfirmationView(TemplateView):
    """To display confirmation"""
    template_name = 'simple/confirmation.html'


class SimpleView(TemplateView):
    """To display simple"""
    template_name = 'simple/simple.html'

    def get(self, request, *args, **kwargs):
        """Get the mail from url"""
        mail = request.GET['mail']
        create_user_consent_information(request, mail=mail)

        return super(SimpleView, self).get(request, *args, **kwargs)


home_view = HomeView.as_view()
confirmation_view = ConfirmationView.as_view()
simple_view = SimpleView.as_view()
